﻿using System;
using System.IO;
using System.Net;
using System.ServiceModel;

namespace RockyJupiter.Mobilengine.Service
{
    //Implement the service interface
    class WdxService: IWdx
    {
        private static string LastDacsid;
        
        //Key used to authenticate Mobilengine to the web service when sending data
        //This key can be specified on the Mobilengine Developer Console
        private static string keyAstroOut = "Very secret key";
        
        //Key used to authenticate RockyJupiter to Mobilengine when downloading images
        //This key is generated by Mobilengine
        private static string keyImageDownload = "808cc2ec1c4446ea9defb02d6e8c5a05";

        private static string imageDownloadBaseUrl = "https://crimson:4443/services/wdx/rdtbin/";

        public void EnqueueDacs(Dacs dacs)
        {
            //Return EnqueDacsFail on key mismatch by throwing the appropriate FaultException
            if (dacs.Key != keyAstroOut)
            {
                Console.WriteLine("Authentication failed for key {0}", dacs.Key);
                throw new FaultException<EnqueueDacsFail>(new EnqueueDacsFail { dacsid = dacs.dacsid, Message = "Dacs authentication failed" });
            }
            
            //Ignore the message if it is a duplicate
            if (dacs.dacsid == LastDacsid)
            {
                Console.WriteLine("Dacs {0} has already been processed, ignoring.");
                return;
            }
            //Store the package ID of the last received message
            LastDacsid = dacs.dacsid;
            
            //Write the contents of the incoming message to the console
            Console.WriteLine("Processing dacs {0}:", dacs.dacsid);
            var statusUpdate = dacs.Content.Item;
            Console.WriteLine(
                "At {0} driver {1} changed the status of assignment {2} from {3} to {4}.",
                statusUpdate.changedAt,
                statusUpdate.driver.Value,
                statusUpdate.assignmentId,
                statusUpdate.oldStatus,
                statusUpdate.newStatus);

            //Download photos if any
            if (dacs.Content.Item.photos != null)
            {
                foreach (var photo in dacs.Content.Item.photos)
                {
                    DownloadImage(photo);
                }
            }

            //Download signature image if there's one
            if (dacs.Content.Item.signature != null)
            {
                DownloadImage(dacs.Content.Item.signature);
            }
        }

        private void DownloadImage(string mediaid)
        {
            var webClient = new WebClient();
            webClient.BaseAddress = imageDownloadBaseUrl;
            //Calculate output image path based on the media id
            var imagePath = Path.GetFullPath(mediaid + ".jpg");
            
            //Download the image
            webClient.DownloadFile(string.Format("{0}?key={1}", mediaid, keyImageDownload), imagePath);
            Console.WriteLine("Image downloaded to {0}.", imagePath);
        } 
        
    }
}
